# businesslive.co.za
# ft.com
# reuters.com
# bbc.com
# theguardian.com
# thetimes.co.uk
# wsj.com
# bloomberg.com
# economist.com
# nytimes.com

def main():
    pass


if __name__ == "__main__":
    main()
